
ComputeSparseGraph <- function(adjmatrix,TINY=1e-5){
    adjmatrixvalues <- as.numeric(names(table(adjmatrix)))
    diag(adjmatrix) <- 0 # just to be safe
    if((length(adjmatrixvalues)==1)&&(adjmatrixvalues!=0)) {print("adjmatrix should only takes values 0, 1 - condition 1")}
    if(length(adjmatrixvalues)>2) {print("adjmatrix should only takes values 0, 1 - condition 2")}
    if((length(adjmatrixvalues)==2)&&(sum(is.element(adjmatrixvalues,c(0,1)))!=2)) {print("adjmatrix should only takes values 0, 1 - condition 3")}
    if(nrow(adjmatrix)!=ncol(adjmatrix)) {print("adjmatrix should be square matrix")}
    if(sum(abs(t(adjmatrix)-adjmatrix))>TINY) {print("adjmatrix should be symmetric")}
    p <- nrow(adjmatrix)
    if((length(adjmatrixvalues)==1)&&(adjmatrixvalues==0)){
        NodeDegree <- rep(0,p)
        NodeWeight <- rep(0,p)
        TotalDegree <- 1 # to prevent failture in Fortran code
        EdgeList <- matrix(0,1,2)
        EdgeListStart <- rep(0,p)
    } else {
        TotalDegree <- sum(adjmatrix)
        NodeDegree <- apply(adjmatrix,2,sum)
        NonIsolatedNodes <- which(NodeDegree!=0)
        NodeWeight <- rep(0,p)
        NodeWeight[NonIsolatedNodes] <- 1/sqrt(NodeDegree[NonIsolatedNodes])
        EdgeList <- NULL
        for(i in 1:p){
            for(j in 1:p){
                if(adjmatrix[i,j]!=0){
                    EdgeList <- rbind(EdgeList,c(i,j))
                }
            }
        }
        EdgeListStart <- rep(0,p)
        for(i in 1:length(NonIsolatedNodes)){
            EdgeListStart[NonIsolatedNodes[i]] <- min(which(EdgeList[,1]==NonIsolatedNodes[i]))
        }
    }
    
    return(list(EdgeList=EdgeList,EdgeListStart=EdgeListStart,NodeDegree=NodeDegree,
                TotalDegree=TotalDegree,NodeWeight=NodeWeight))
}

ComputeSparseGraph_SparseInput <- function(EdgeListx,EdgeListy,p,TINY=1e-5){
    # have not considered the case where EdgeListx and EdgeListy are empty or NA
    if(length(EdgeListx)!=length(EdgeListy)) {print("EdgeListx and EdgeListy have different length, error!")}
    EdgeList1 <- cbind(EdgeListx=EdgeListx,EdgeListy=EdgeListy)
    EdgeList2 <- cbind(EdgeListx=EdgeListy,EdgeListy=EdgeListx)
    EdgeList <- unique(rbind(EdgeList1,EdgeList2))
    TotalDegree <- length(EdgeListx)
    NodeDegree <- rep(0,p)
    NodeDegree[as.numeric(names(table(sort(c(EdgeListx,EdgeListy)))))] <- as.numeric(table(sort(c(EdgeListx,EdgeListy))))/2
    NonIsolatedNodes <- which(NodeDegree!=0)
    NodeWeight <- rep(0,p)
    NodeWeight[NonIsolatedNodes] <- 1/sqrt(NodeDegree[NonIsolatedNodes])
    EdgeListStart <- rep(0,p)
    for(i in 1:length(NonIsolatedNodes)){
            EdgeListStart[NonIsolatedNodes[i]] <- min(which(EdgeList[,1]==NonIsolatedNodes[i]))
    }
    
    return(list(EdgeList=EdgeList,EdgeListStart=EdgeListStart,NodeDegree=NodeDegree,
                TotalDegree=TotalDegree,NodeWeight=NodeWeight))
}

##################

GFuseTLPclogit <- function(X,Y,SubjectMap,nlambda1=20,lambda1=NULL,maxlambda1=NULL,
                  nLpMultiplier=1e-2,nGpMultiplier=1e-3,nlambda2=10,lambda2=NULL,ntau=1,tau=NULL,
                  adjmatrix=NULL,EdgeList=NULL,MAXITER=1000,thre=1e-3,maxvarPercent=1,rho=1){
    p <- ncol(X)
    n <- nrow(X)
    n_case <- sum(Y!=0)
    n_control <- sum(Y==0)
    if(n_case != n_control) {print("number of case differ from number of control, error!!!")}
    Xmean <- apply(X,2,mean)
    Xsd <- rep(0.0,p)
    for(j in 1:p){
        X[,j] <- X[,j]-Xmean[j]
        Xsd[j] <- sqrt(sum(X[,j]*X[,j])/n)
        X[,j] <- X[,j]/Xsd[j]
    }

    # fit lasso penalized cond-logistic reg 
    # Note: the calculation for max_lambda for lasso is only correct for 1-1 matched case control study
    tempr <- 1/n*(Y-0.5)
    maxlambdacandidates <- rep(0,p)   
    for(j in 1:p){
        maxlambdacandidates[j] <- sum(tempr*X[,j])
    }
    maxlassolambda <- max(abs(maxlambdacandidates))
    minlassolambdamultiplier <- (ifelse(n < p, nLpMultiplier, nGpMultiplier))^(1/(nlambda1-1))
    lassolambda <- maxlassolambda*minlassolambdamultiplier^(seq(0,nlambda1-1))
    lassolambda <- sort(lassolambda,decreasing=T)    
    lambda_length <- length(lassolambda)
    eps <- thre
    
    EST_lassobeta <- rep(0.0,p*nlambda1)
    LassoclogisticObj <- .Fortran("Lassoclogistic_Ffunction",n=as.integer(n),n_case=as.integer(n_case),n_control=as.integer(n_control),
                                    p=as.integer(p),X=as.double(X),Xsd=as.double(Xsd),Y=as.double(Y),SubjectMap=as.integer(SubjectMap),
                                    lambda=as.double(lassolambda),lambda_length=as.integer(lambda_length),
                                    MAXITER=as.integer(MAXITER),eps=as.double(thre),maxvarPercent=as.double(maxvarPercent),
                                    EST_BETA=as.double(EST_lassobeta))
    EST_lassobeta <- (LassoclogisticObj$EST_BETA)[1:(p*lambda_length)]
    EST_lassobetamatrix <- matrix(as.numeric(EST_lassobeta),p,lambda_length)
    lassotmin <- min(abs(EST_lassobetamatrix[abs(EST_lassobetamatrix)>1e-6]))
    lassotmax <- max(abs(EST_lassobetamatrix[abs(EST_lassobetamatrix)>1e-6])) 
    # print(paste("lassotmax is",lassotmax))
    
    if(!is.null(adjmatrix)){
            SparseGraphObj <- ComputeSparseGraph(adjmatrix)
            EdgeList <- SparseGraphObj$EdgeList
            NodeDegree <- SparseGraphObj$NodeDegree
            TotalDegree <- SparseGraphObj$TotalDegree
            NodeWeight <- SparseGraphObj$NodeWeight  
    } else {
        if(!is.null(EdgeList)){
            EdgeListx = EdgeList[,1]
            EdgeListy = EdgeList[,2]
            SparseGraphObj <- ComputeSparseGraph_SparseInput(EdgeListx,EdgeListy,p)
            EdgeList <- SparseGraphObj$EdgeList
            NodeDegree <- SparseGraphObj$NodeDegree
            TotalDegree <- SparseGraphObj$TotalDegree
            NodeWeight <- SparseGraphObj$NodeWeight  
        } else {
            print("in GFuseTLP_clogistic_Rfunction, provide either adjmatrix, or EdgeList")
        }
    } 
    
    if(is.null(tau)){
        tau <- lassotmax
    }
    tau <- sort(tau)
                      
    if(is.null(lambda1)){
        if(!is.null(maxlambda1)){
            minlambdamultiplier <- (ifelse(n < p, nLpMultiplier, nGpMultiplier))^(1/(nlambda1-1))
            lambda1 <- maxlambda1*minlambdamultiplier^(seq(0,nlambda1-1))    
        } else {
            lambda1 <- exp(seq(log(sqrt(p)*lassotmax/4),log(lassotmax/10),length.out=nlambda1))
        }
    }
    lambda1 <- sort(lambda1*tau,decreasing=TRUE)
    lambda2 <- exp(seq(log(lassotmax*tau/sqrt(TotalDegree/2)),log(lassotmax*tau*sqrt(TotalDegree/2)),length.out=nlambda2))
    
    lambda1_length <- length(lambda1)
    lambda2_length <- length(lambda2)
    tau_length <- length(tau)
    
    EST_beta <- rep(0.0,p*lambda1_length*lambda2_length*tau_length)
    EST_Q1 <- rep(0.0,p*lambda1_length*lambda2_length*tau_length)

    GFuseTLPclogisticObj <- .Fortran("GFuseTLPclogistic_Ffunction",n=as.integer(n),n_case=as.integer(n_case),n_control=as.integer(n_control),
                                    p=as.integer(p),X=as.double(X),Xsd=as.double(Xsd),Y=as.double(Y),SubjectMap=as.integer(SubjectMap),
                                    EdgeList=as.integer(EdgeList),NodeDegree=as.integer(NodeDegree),
                                    TotalDegree=as.integer(TotalDegree),NodeWeight=as.double(NodeWeight),
                                    lambda1=as.double(lambda1),lambda1_length=as.integer(lambda1_length),
                                    lambda2=as.double(lambda2),lambda2_length=as.integer(lambda2_length),
                                    tau=as.double(tau),tau_length=as.integer(tau_length),
                                    MAXITER=as.integer(MAXITER),eps=as.double(thre),maxvarPercent=as.double(maxvarPercent),
                                    rho=as.double(rho),EST_BETA=as.double(EST_beta),EST_Q1=as.double(EST_Q1))   
                                                                
    EST_beta <- (GFuseTLPclogisticObj$EST_BETA)[1:(p*lambda1_length*lambda2_length*tau_length)]
    EST_beta_matrix <- matrix(as.numeric(EST_beta),p,lambda1_length*lambda2_length*tau_length)
    
    EST_Q1 <- (GFuseTLPclogisticObj$EST_Q1)[1:(p*lambda1_length*lambda2_length*tau_length)]
    EST_Q1_matrix <- matrix(as.numeric(EST_Q1),p,lambda1_length*lambda2_length*tau_length)
    
    results <- list(beta=EST_Q1_matrix,lambda1=lambda1,lambda2=lambda2,tau=tau)
    class(results) <- "tlpclogit"  
    return(results)
}
 