\name{GFuseTLPclogit}
\alias{GFuseTLPclogit}
\title{GFuseTLP penalized conditional logistic regression model}
\description{
    Fit GFuseTLP penalized conditional logistic regression model for 
    high-dimensional one-to-one mathced case-control data.
    }
\usage{GFuseTLPclogit(X,Y,SubjectMap,nlambda1=20,lambda1=NULL,maxlambda1=NULL,
    nLpMultiplier=1e-2,nGpMultiplier=1e-3,nlambda2=10,lambda2=NULL,
    ntau=1,tau=NULL,adjmatrix=NULL,EdgeList=NULL,MAXITER=1000,thre=1e-3,
    maxvarPercent=1,rho=1)
    }    
\arguments{
    \item{X}{The covariance matrix (\eqn{n} x \eqn{p}) without an intercept.}
    \item{Y}{The dichotomous outcome variable, with value 1 for cases and 0 for 
        the matched controls.}
    \item{SubjectMap}{A vector of consecutive integers indicating the stratum 
        of each observation. Each stratum must have one case and one control.}
    \item{nlambda1}{The number of \code{lambda1} values, and default is 20.}
    \item{lambda1}{A user supplied sequence of \code{lambda1} values. If 
        unspecified, the program computes \code{lambda1} sequence.}    
    \item{maxlambda1}{A user supplied number of the maximum \code{lambda1} value.}
    \item{nLpMultiplier}{Smallest value for \code{lambda1} as a fraction of maximum
        \code{lambda1} when \eqn{n} is less than \eqn{p}, and default is 1e-2.}
    \item{nGpMultiplier}{Smallest value for lambda as a fraction of maximum
        \code{lambda1} when \eqn{n} is greater than \eqn{p}, and default 
        is 1e-3.}       
    \item{nlambda2}{The number of \code{lambda2} values, and default is 10.}
    \item{lambda2}{A user supplied sequence of \code{lambda2} values. If 
        unspecified, the program computes \code{lambda2} sequence.}
    \item{ntau}{The number of \code{tau} values and default is 1.}
    \item{tau}{A user supplied sequence of \code{tau} values. If 
        unspecified, and the program computes \code{tau}.} 
    \item{adjmatrix}{A user supplied adjacency matrix (\eqn{p} x \eqn{p}).}    
    \item{EdgeList}{A user supplied adjacency list as a two column matrix.} 
    \item{MAXITER}{Maximum number of iterations and default is 1000.}
    \item{thre}{Convergence threshold and default is \code{1E-3}.}
    \item{maxvarPercent}{Limit the maximum precentage of variables ever to 
        be nonzero.}
    \item{rho}{Non-negative dual update step length in the algorithm, and
        default is 1.}
        }
\details{               
       The penalty function of \code{GFuseTLPclogit} is defined as
       \deqn{\lambda_1\sum\min(\frac{|\beta|}{\tau},1)+\lambda_2\sum_{u\sim v}|\frac{|\beta_u|}{\sqrt{d_u}}-\frac{|\beta_v|}{\sqrt{d_v}}|,}
       where the edge information is contained either in \code{adjmatrix} or
       in \code{EdgeList}. This penalty is equivalent to the Lasso penalty when
       \code{lambda2=0} and \code{tau} is sufficiently large.
       }
\value{
    \item{beta}{The coefficient matrix (\eqn{p} x \code{nlambda})}
    \item{lambda1}{The sequence of \code{lambda1} values used}
    \item{lambda2}{The sequence of \code{lambda2} values used}
    \item{tau}{The sequence of \code{tau} used}
     }
\references{
    Zhou, H., Wang,S., and Zheng, T. (2014)
    \emph{Penalized conditional logistic regression for high-dimensional 
    matched case-control DNA methylation data}, manuscript
    }
\author{Hui Zhou <hz2240@columbia.edu>}
\examples{
    n <- 200
    p <- 200
    
    # one-to-one matched set
    y <- c(rep(1,n/2),rep(0,n/2))
    submap <- rep(seq(1,n/2),2)
    
    # an example of covariance matrix for matched case-control data
    x <- matrix(0,n,p)
    BETA <- c(rep(c(1,-1),5),rep(0,p-10))
    groupsize <- 10
    Sigma <- matrix(0,p,p)
    for(ngrp in 1:(p/groupsize))
      Sigma[((ngrp-1)*groupsize+1):(ngrp*groupsize),
      ((ngrp-1)*groupsize+1):(ngrp*groupsize)] <- 0.5
    diag(Sigma) <- 1
    chol_Sigma <- chol(Sigma)
    for(i in 1:(n/2)){ 
        tempx1 <- as.numeric(rnorm(p,0,1)\%*\%chol_Sigma)
        tempx2 <- as.numeric(rnorm(p,0,1)\%*\%chol_Sigma)
        tempyprob <- exp(tempx1\%*\%BETA)/( exp(tempx1\%*\%BETA)+exp(tempx2\%*\%BETA))
        tempy <- rbinom(1,1,tempyprob)
        if(tempy==1){
            x[i,] <- tempx1;x[i+(n/2),] <- tempx2
        } else {
            x[i,] <- tempx2;x[i+(n/2),] <- tempx1
        }
    }

    # an example of adjacency matrix
    adjm <- (Sigma!=0)*1
    diag(adjm)<-0

    # GFuseTLP
    fm <- GFuseTLPclogit(X=x,Y=y,SubjectMap=submap,adjmatrix=adjm)

}

